# with secant
# vertical position of a thrown ball
# d^2x/dt^2=-g
# b.c x=0 at time t=0 and t=10
# set dx/dt=v then dv/dt=-g
import numpy as np
import matplotlib.pyplot as plt

def f(r, t):
  x=r[0]
  v=r[1]
  return np.array([v, -G], float)

def RK4(f, x, t, h):
  c=np.zeros((4, 2), float)
  c[0]=h*f(x, t)
  c[1]=h*f(x+c[0]/2, t+h/2)
  c[2]=h*f(x+c[1]/2, t+h/2)
  c[3]=h*f(x+c[2], t+h)
  return x+(c[0]+2*c[1]+2*c[2]+c[3])/6

def g(v, t):
  r=np.array([0., v], float)
  h=t[1]-t[0]
  for i in range(len(t)-1):
    r=RK4(f, r, t[i], h)
  return r[0]-0.
  

if __name__=="__main__":
  G=9.81
  t_i=0.; t_f=10.
  t=np.linspace(t_i, t_f, 1000)
  
  # secant
  tolerance=1e-6
  v1=0.001
  v2=1.
  h1=g(v1, t)
  
  while abs(v2-v1)>tolerance:
    h2=g(v2, t)
    dv=v2-v1
    dh=h2-h1
    v1=v2
    v2-=dv*h2/dh
    h1=h2
  
  print("Needs Velocity: ", v_i, sep="")
  
  x=np.zeros_like(t, float)
  v=np.zeros_like(t, float)
  v[0]=v_i
  
  r=np.array([0, v_i], float)
  h=t[1]-t[0]
  for i in range(1, len(t)):
    r=RK4(f, r, t[i-1], h)
    x[i]=r[0]
    v[i]=r[1]
  
  plt.plot(t, x)
  plt.show()
